/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "chartwindow.h"
#include "ui_chartwindow.h"
#include "settings.h"

#include <QFileDialog>
#include <QtDebug>
#include <QPen>
#include <QBrush>
#include <QMessageBox>
#include <QInputDialog>
#include <QMenu>
#include <QIcon>

/*
 * This was adapted from the zxweather desktop client where the
 * chart window is somewhat more interactive and is able to handle
 * more than just one temperature graph.
 */


ChartWindow::ChartWindow(SampleSet samples, SampleColumns columns,
                         QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ChartWindow)
{
    ui->setupUi(this);

    gridVisible = true;

    basicInteractionManager.reset(
            new BasicQCPInteractionManager(ui->chart, this));

    // Chart events
    connect(ui->chart, SIGNAL(titleDoubleClick(QMouseEvent*,QCPPlotTitle*)),
            this, SLOT(titleDoubleClick(QMouseEvent*, QCPPlotTitle*)));
    connect(ui->chart,
            SIGNAL(axisDoubleClick(QCPAxis*,
                                   QCPAxis::SelectablePart,
                                   QMouseEvent*)),
            this,
            SLOT(axisDoubleClick(QCPAxis*,
                                 QCPAxis::SelectablePart,
                                 QMouseEvent*)));
    connect(ui->chart, SIGNAL(customContextMenuRequested(QPoint)),
            this, SLOT(chartContextMenuRequested(QPoint)));


    bool temp = columns.testFlag(SC_Temperature);
    bool dp = columns.testFlag(SC_DewPoint);
    bool hum = columns.testFlag(SC_Humidity);

    setWindowTitle(tr("Chart"));

    // Plot the chart
    ui->chart->clearGraphs();
    ui->chart->clearPlottables();

    // Setup the first value axis.
    if (temp || dp) {
        ui->chart->yAxis->setTickLabels(true);
        ui->chart->yAxis->setLabel(tr("Temperature (°C)"));
        ui->chart->yAxis->grid()->setVisible(true);
    } else if (hum) {
        ui->chart->yAxis->setTickLabels(true);
        ui->chart->yAxis->setLabel(tr("Humidity (%)"));
        ui->chart->yAxis->grid()->setVisible(true);
    }

    // Setup the second value axis if necessary
    if ((temp || dp) && hum) {
        if (ui->chart->yAxis2 == 0) {
            ui->chart->axisRect()->addAxis(QCPAxis::atRight);
        } else {
            ui->chart->yAxis2->setVisible(true);
        }

        ui->chart->yAxis2->setTickLabels(true);
        ui->chart->yAxis2->setLabel(tr("Humidity (%)"));
        ui->chart->yAxis2->grid()->setVisible(true);
    }

    ui->chart->xAxis->setLabel(tr("Time"));
    ui->chart->xAxis->setTickLabelType(QCPAxis::ltDateTime);

    ChartColours colours = Settings::getInstance().getChartColours();

    if (temp) {
        QCPGraph* graph = ui->chart->addGraph();
        graph->setData(samples.timestamp, samples.temperature);
        graph->setName(tr("Temperature"));
        graph->setPen(QPen(colours.temperature));
    }

    if (dp) {
        QCPGraph* graph = ui->chart->addGraph();
        graph->setData(samples.timestamp, samples.dew_point);
        graph->setName(tr("Dew Point"));
        graph->setPen(QPen(colours.dewPoint));
    }

    if (hum) {
        QCPGraph* graph = ui->chart->addGraph();
        graph->setData(samples.timestamp, samples.humidity);
        graph->setName(tr("Humidity"));
        graph->setPen(QPen(colours.humidity));

        // If we've got temperature in the plot then humidity goes on the
        // second (right-hand) value axis.
        if (temp || dp) {
            graph->setValueAxis(ui->chart->yAxis2);
        }
    }

    basicInteractionManager->setYAxisLockEnabled(true);

    ui->chart->rescaleAxes();
    ui->chart->replot();

}

ChartWindow::~ChartWindow()
{
    delete ui;
}

void ChartWindow::axisDoubleClick(QCPAxis* axis, QCPAxis::SelectablePart part,
                                  QMouseEvent* /*event*/)
{
    // If the user double-clicked on the axis label then ask for new
    // label text.
    if (part == QCPAxis::spAxisLabel) {
        bool ok;

        QString newLabel = QInputDialog::getText(
                    this,
                    QString(tr("Axis Label")),
                    tr("New axis label:"),
                    QLineEdit::Normal,
                    axis->label(),
                    &ok);
        if (ok) {
            axis->setLabel(newLabel);
            ui->chart->replot();
        }
    }
}

void ChartWindow::titleDoubleClick(QMouseEvent *event, QCPPlotTitle *title)
{
    Q_UNUSED(event)

    // Allow the graph title to be changed.
    bool ok;
    plotTitleValue = QInputDialog::getText(
                this,
                tr("Chart Title"),
                tr("New chart title:"),
                QLineEdit::Normal,
                title->text(),
                &ok);
    if (ok) {
        title->setText(plotTitleValue);
        ui->chart->replot();
    }
}

void ChartWindow::chartContextMenuRequested(QPoint point)
{

    if (ui->chart->legend->selectTest(point, false) >= 0) {
        showLegendContextMenu(point);
        return;
    }

    QMenu* menu = new QMenu(this);
    menu->setAttribute(Qt::WA_DeleteOnClose);

    // As there is no more separate save button, we put the save
    // option here.
    menu->addAction(tr("Save As..."),
                             this, SLOT(save()));

    /******** Plot feature visibility ********/
    menu->addSeparator();

    // Title visibility option.
    QAction* action = menu->addAction(tr("Show Title"),
                             this, SLOT(showTitleToggle()));
    action->setCheckable(true);
    action->setChecked(!plotTitle.isNull());


    // Legend visibility option.
    action = menu->addAction(tr("Show Legend"),
                             this, SLOT(showLegendToggle()));
    action->setCheckable(true);
    action->setChecked(ui->chart->legend->visible());

    // Grid visibility option
    action = menu->addAction(tr("Show Grid"),
                             this,
                             SLOT(showGridToggle()));
    action->setCheckable(true);
    action->setChecked(gridVisible);

    menu->addSeparator();
    action = menu->addAction(tr("Lock &Y Axis"),
                             this, SLOT(toggleYAxisLock()));
    action->setCheckable(true);
    action->setChecked(basicInteractionManager->isYAxisLockEnabled());

    /******** Finished ********/
    menu->popup(ui->chart->mapToGlobal(point));
}

void ChartWindow::addTitle()
{
    bool ok;
    if (plotTitleValue.isNull()) {
        // Title has never been set. Ask for a value.
        plotTitleValue = QInputDialog::getText(
                    this,
                    tr("Chart Title"),
                    tr("New chart title:"),
                    QLineEdit::Normal,
                    "",
                    &ok);
    } else {
        // Just re-use the previous title.
        ok = true;
    }

    if (ok) {
        ui->chart->plotLayout()->insertRow(0);
        plotTitle = new QCPPlotTitle(ui->chart, plotTitleValue);
        ui->chart->plotLayout()->addElement(0, 0, plotTitle);
        ui->chart->replot();
    }
}

void ChartWindow::removeTitle()
{
    ui->chart->plotLayout()->remove(plotTitle);
    ui->chart->plotLayout()->simplify();
    ui->chart->replot();
}

void ChartWindow::showLegendToggle()
{
    ui->chart->legend->setVisible(!ui->chart->legend->visible());
    ui->chart->replot();
}

void ChartWindow::showTitleToggle()
{
    if (plotTitle.isNull())
        addTitle();
    else
        removeTitle();
}

void ChartWindow::showGridToggle() {

    gridVisible = !gridVisible;

    QList<QCPAxis*> axes = valueAxes();
    axes << ui->chart->xAxis;

    foreach(QCPAxis* axis, axes) {
        axis->grid()->setVisible(gridVisible);
    }
    ui->chart->replot();
}

void ChartWindow::moveLegend()
{
    if (QAction* menuAction = qobject_cast<QAction*>(sender())) {
        // We were called by a context menu. We should have the necessary
        // data.
        bool ok;
        int intData = menuAction->data().toInt(&ok);
        if (ok) {
            ui->chart->axisRect()->insetLayout()->setInsetAlignment(
                        0, (Qt::Alignment)intData);
            ui->chart->replot();
        }
    }
}


QList<QCPAxis*> ChartWindow::valueAxes() {
    return ui->chart->axisRect()->axes(QCPAxis::atLeft | QCPAxis::atRight);
}

void ChartWindow::showLegendContextMenu(QPoint point)
{
    QMenu *menu = new QMenu(this);
    menu->setAttribute(Qt::WA_DeleteOnClose);

    // Options to re-position the legend
    menu->addAction(tr("Move to top left"),
                    this,
                    SLOT(moveLegend()))->setData((int)(Qt::AlignTop
                                                       | Qt::AlignLeft));
    menu->addAction(tr("Move to top center"),
                    this,
                    SLOT(moveLegend()))->setData((int)(Qt::AlignTop
                                                       | Qt::AlignHCenter));
    menu->addAction(tr("Move to top right"),
                    this,
                    SLOT(moveLegend()))->setData((int)(Qt::AlignTop
                                                       | Qt::AlignRight));
    menu->addAction(tr("Move to bottom right"),
                    this,
                    SLOT(moveLegend()))->setData((int)(Qt::AlignBottom
                                                       | Qt::AlignRight));
    menu->addAction(tr("Move to bottom center"),
                    this,
                    SLOT(moveLegend()))->setData((int)(Qt::AlignBottom
                                                       | Qt::AlignHCenter));
    menu->addAction(tr("Move to bottom left"),
                    this,
                    SLOT(moveLegend()))->setData((int)(Qt::AlignBottom
                                                       | Qt::AlignLeft));

    // And an option to get rid of it entirely.
    menu->addSeparator();
    menu->addAction(tr("Hide"), this, SLOT(showLegendToggle()));

    menu->popup(ui->chart->mapToGlobal(point));
}

void ChartWindow::save() {

    QString pdfFilter = tr("Adobe Portable Document Format") + " (*.pdf)";
    QString pngFilter = tr("Portable Network Graphics") + " (*.png)";
    QString jpgFilter = tr("JPEG") + " (*.jpg)";
    QString bmpFilter = tr("Windows Bitmap") + " (*.bmp)";

    QString filter = pngFilter + ";;" + pdfFilter + ";;" +
            jpgFilter + ";;" + bmpFilter;

    QString selectedFilter;

    QString fileName = QFileDialog::getSaveFileName(
                this, tr("Save As") ,"", filter, &selectedFilter);


    qDebug() << selectedFilter;
    qDebug() << fileName;

    // To prevent selected stuff appearing in the output
    ui->chart->deselectAll();

    // TODO: Provide a UI to ask for width, height, cosmetic pen (PS), etc.
    if (selectedFilter == pdfFilter)
        ui->chart->savePdf(fileName);
    else if (selectedFilter == pngFilter)
        ui->chart->savePng(fileName);
    else if (selectedFilter == jpgFilter)
        ui->chart->saveJpg(fileName);
    else if (selectedFilter == bmpFilter)
        ui->chart->saveBmp(fileName);
}

void ChartWindow::toggleYAxisLock() {

    bool isEnabled = basicInteractionManager->isYAxisLockEnabled();

    basicInteractionManager->setYAxisLockEnabled(!isEnabled);

    ui->chart->deselectAll();
    ui->chart->replot();
}

