/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "exportdialog.h"
#include "ui_exportdialog.h"
#include "settings.h"

#include <QFileDialog>
#include <QMessageBox>
#include <QProgressDialog>

ExportDialog::ExportDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ExportDialog)
{
    ui->setupUi(this);

    ui->startTime->setDateTime(QDateTime::currentDateTime().addDays(-1));
    ui->endTime->setDateTime(QDateTime::currentDateTime());

    // Time ranges
    connect(ui->rbTCustom, SIGNAL(clicked()), this, SLOT(dateChanged()));
    connect(ui->rbTThisMonth, SIGNAL(clicked()), this, SLOT(dateChanged()));
    connect(ui->rbTThisWeek, SIGNAL(clicked()), this, SLOT(dateChanged()));
    connect(ui->rbTThisYear, SIGNAL(clicked()), this, SLOT(dateChanged()));
    connect(ui->rbTToday, SIGNAL(clicked()), this, SLOT(dateChanged()));
    connect(ui->rbTYesterday, SIGNAL(clicked()), this, SLOT(dateChanged()));

    // Delimiter types
    connect(ui->rbCommaDelimited, SIGNAL(clicked()),
            this, SLOT(delimiterTypeChanged()));
    connect(ui->rbTabDelimited, SIGNAL(clicked()),
            this, SLOT(delimiterTypeChanged()));
    connect(ui->rbOtherDelimiter, SIGNAL(clicked()),
            this, SLOT(delimiterTypeChanged()));

    // ok button
    connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(exportData()));
}

ExportDialog::~ExportDialog()
{
    delete ui;
}

void ExportDialog::dateChanged() {
    if (ui->rbTCustom->isChecked()) {
        ui->startTime->setEnabled(true);
        ui->endTime->setEnabled(true);
    } else {
        ui->startTime->setEnabled(false);
        ui->endTime->setEnabled(false);
    }
}

void ExportDialog::delimiterTypeChanged() {
    if (ui->rbOtherDelimiter->isChecked())
        ui->leCustomDelimiter->setEnabled(true);
    else
        ui->leCustomDelimiter->setEnabled(false);
}

QString ExportDialog::getDelimiter() {
    QString delimiter;
    if (ui->rbCommaDelimited->isChecked())
        delimiter = ",";
    else if (ui->rbTabDelimited->isChecked())
        delimiter = "\t";
    else if (ui->rbOtherDelimiter->isChecked())
        delimiter = ui->leCustomDelimiter->text();

    return delimiter;
}

QDateTime ExportDialog::getStartTime() {
    QDateTime time = QDateTime::currentDateTime();
    time.setTime(QTime(0,0));

    int year = time.date().year();
    int month = time.date().month();

    if (ui->rbTToday->isChecked())
        return time;
    else if (ui->rbTYesterday->isChecked()) {
        time.setDate(time.date().addDays(-1));
        return time;
    } else if (ui->rbTThisWeek->isChecked()) {
        int subtractDays = 1 - time.date().dayOfWeek();
        time.setDate(time.date().addDays(subtractDays));
        return time;
    } else if (ui->rbTThisMonth->isChecked()) {
        time.setDate(QDate(year, month, 1));
        return time;
    } else if (ui->rbTThisYear->isChecked()) {
        time.setDate(QDate(year,1,1));
        return time;
    } else
        return ui->startTime->dateTime();
}

QDateTime ExportDialog::getEndTime() {
    QDateTime time = QDateTime::currentDateTime();
    time.setTime(QTime(23,59,59));

    int year = time.date().year();
    int month = time.date().month();

    if (ui->rbTToday->isChecked()) {
        return time;
    } else if (ui->rbTYesterday->isChecked()) {
        time.setDate(time.date().addDays(-1));
        return time;
    } else if (ui->rbTThisWeek->isChecked()) {
        int addDays = 7 - time.date().dayOfWeek();
        time.setDate(time.date().addDays(addDays));
        return time;
    } else if (ui->rbTThisMonth->isChecked()) {
        QDate date(year,month,1);
        date = date.addMonths(1);
        date = date.addDays(-1);
        time.setDate(date);
        return time;
    } else if (ui->rbTThisYear->isChecked()) {
        time.setDate(QDate(year,12,31));
        return time;
    } else
        return ui->endTime->dateTime();
}

SampleColumns ExportDialog::getColumns()
{
    SampleColumns columns;


    columns |= SC_Timestamp;

    if (ui->cbTemperature->isChecked())
        columns |= SC_Temperature;
    if (ui->cbHumidity->isChecked())
        columns |= SC_Humidity;
    if (ui->cbDewPoint->isChecked())
        columns |= SC_DewPoint;

    return columns;
}

bool ExportDialog::includeHeader()
{
    return ui->cbIncludeHeadings->isChecked();
}


void ExportDialog::exportData() {
    QString delimiter = getDelimiter();

    QString filterList =
            tr("Data file (*.dat);;Comma separated values (*.csv);;Text file (*.txt)");

    QStringList filters = QString(filterList).split(";;");
    QString selectedFilter = filters.first();

    if (delimiter == ",")
        selectedFilter = filters.at(1); // csv

    QString filename = QFileDialog::getSaveFileName(this,
                                                    tr("Export data..."),
                                                    QString(),
                                                    filterList,
                                                    &selectedFilter);
    if (filename.isEmpty()) {
        reject(); // user canceled
        return;
    }

    targetFilename = filename;

    accept();
}

void ExportDialog::performExport(SampleSet samples,
                                 QWidget *parent,
                                 SampleColumns columns,
                                 QString targetFileName,
                                 QString delimiter,
                                 bool includeHeader)
{
    int sampleCount = samples.timestamp.count();
    QProgressDialog progressDialog(parent);
    progressDialog.setWindowTitle(tr("Exporting Data..."));
    progressDialog.setMaximum(sampleCount);

    QFile dataFile(targetFileName);

    if (!dataFile.open(QIODevice::WriteOnly)) {
        QMessageBox::critical(parent, tr("Error opening file"),
                              QString(tr("Failed to open file for writing. %s").arg(
                                          dataFile.errorString())));
        return;
    }

    if (includeHeader) {
        QString headerRow = ExportDialog::getHeaderRow(columns, delimiter);
        if (!headerRow.isEmpty())
            dataFile.write(headerRow.toLocal8Bit());
    }

    for (int i = 0; i < samples.timestamp.count(); i++) {
        QStringList rowData;

        if (columns.testFlag(SC_Timestamp))
            rowData.append(QDateTime::fromTime_t(
                               samples.timestampUnix.at(i))
                           .toString(Qt::ISODate));
        if (columns.testFlag(SC_Temperature))
            rowData.append(QString::number(samples.temperature.at(i),'f', 1));

        if (columns.testFlag(SC_Humidity))
            rowData.append(QString::number(samples.humidity.at(i),'f', 1));

        if (columns.testFlag(SC_DewPoint))
            rowData.append(QString::number(samples.dew_point.at(i),'f', 1));

        QString row = rowData.join(delimiter) + "\n";
        dataFile.write(row.toLocal8Bit());

        // Only update on every 25th row
        if (i % 25 == 0) {
            progressDialog.setValue(i);

            if (progressDialog.wasCanceled()) {
                dataFile.close();
                return;
            }
        }

    }
    dataFile.close();
    progressDialog.reset();
}

QString ExportDialog::getHeaderRow(SampleColumns columns, QString delimiter) {
    QStringList columnNames;

    if (columns.testFlag(SC_Timestamp))
        columnNames.append(tr("Timestamp"));
    if (columns.testFlag(SC_Temperature))
        columnNames.append(tr("Temperature"));
    if (columns.testFlag(SC_Humidity))
        columnNames.append(tr("Humidity"));
    if (columns.testFlag(SC_DewPoint))
        columnNames.append(tr("Dew Point"));

    return columnNames.join(delimiter) + "\n";
}


