/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QScopedPointer>
#include <QSystemTrayIcon>
#include <QMenu>
#include <QPointer>
#include <QAction>

#include "qcp/qcustomplot.h"

#include "abstractdatasource.h"
#include "abstractdatalogger.h"
#include "sampleset.h"
#include "samplecolumns.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private slots:
    void dataArrived(float temperature);
    void dataArrived(float temperature, float humidity);
    void messageReceived(QString message);
    void dataSourceError(QString title, QString message);

    void about();
    void settings();
    void chart();
    void exportData();

    void quit();

    /**
     * @brief trayIconActivated is called when the user clicks on the system
     * tray icon. It raises the window and gives it focus.
     * @param reason Reason for activation (click, double click, etc).
     */
    void trayIconActivated(QSystemTrayIcon::ActivationReason reason);

protected:
    /**
     * @brief changeEvent handles minimising the window to the system tray if
     * that setting is turned on.
     * @param e details about the event that has occurred.
     */
    void changeEvent(QEvent *e);

    /**
     * @brief closeEvent handles hiding the program to the system tray when the
     * window is closed if the close to system tray option is turned on.
     * @param event details about the event that has occurred.
     */
    void closeEvent(QCloseEvent *event);

private:
    Ui::MainWindow *ui;
    QScopedPointer<AbstractDataSource> dataSource;
    QScopedPointer<AbstractDataLogger> dataLogger;

    QScopedPointer<QSystemTrayIcon> sysTrayIcon;
    QScopedPointer<QMenu> trayIconMenu;
    QScopedPointer<QAction> restoreAction;
    QScopedPointer<QAction> quitAction;

    QPointer<QCPGraph> temperatureGraph;
    QPointer<QCPGraph> temperaturePointGraph;
    QPointer<QCPGraph> humidityGraph;
    QPointer<QCPGraph> humidityPointGraph;
    QPointer<QCPGraph> dewPointGraph;
    QPointer<QCPGraph> dewPointPointGraph;

    bool logData;

    bool humidityAvailable;

    void setupPlot();
    void updatePlot(float temperature);
    void updatePlot(float temperature, float humidity, float dewPoint);
    void setupDataSource();
    void setupLogger();
    SampleSet GetSamples(QDateTime startTime, QDateTime endTime, SampleColumns columns);

    float dewPoint(float temperature, float humidity);

    // Graph averaging stuff
    void averageData(float temperature, float humidity, float dewPoint);
    int samples;
    float temperatureTotal;
    float humidityTotal;
    float dewPointTotal;
    qint64 currentTs;
    qint64 nextTs();
};

#endif // MAINWINDOW_H
