/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "settingsdialog.h"
#include "ui_settingsdialog.h"

#include "settings.h"

#include <QFileDialog>
#include <QStandardPaths>
#include <QMessageBox>

#define PSQL 0
#define SQLITE 1

SettingsDialog::SettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SettingsDialog)
{
    ui->setupUi(this);
    connect(ui->rbUsbSensor, SIGNAL(toggled(bool)), this, SLOT(dataSourceChanged()));
    connect(ui->rbTemperHumSensor, SIGNAL(toggled(bool)), this, SLOT(dataSourceChanged()));
    connect(ui->rbDatabase, SIGNAL(toggled(bool)), this, SLOT(dataSourceChanged()));
    connect(ui->cbLogToDb, SIGNAL(toggled(bool)), this, SLOT(dataSourceChanged()));
    connect(ui->tbBrowseForFile, SIGNAL(clicked()), this, SLOT(browseForDbFile()));
    connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(done()));
    loadSettings();
}

SettingsDialog::~SettingsDialog()
{
    delete ui;
}

void SettingsDialog::loadSettings() {
    Settings& settings = Settings::getInstance();

    // General tab

    ui->cbMinimiseToSysTray->setChecked(settings.miniseToSysTray());
    ui->cbMinimiseToSysTrayOnExit->setChecked(settings.closeToSysTray());

    auto chartColours = settings.getChartColours();
    ui->cbChartLineColour->setColor(chartColours.temperature);
    ui->cbHumidityChartLineColour->setColor(chartColours.humidity);
    ui->cbDewPointChartLineColour->setColor(chartColours.dewPoint);

    auto mainWindowGraphs = settings.getMainWindowGraphsEnabled();
    ui->cbPlotTemperature->setChecked(mainWindowGraphs.temperature);
    ui->cbPlotHumidity->setChecked(mainWindowGraphs.humidity);
    ui->cbPlotDewPoint->setChecked(mainWindowGraphs.dewPoint);
    ui->cbShowLegend->setChecked(settings.mainWindowGraphLegendEnabled());
    ui->cbAveragedGraph->setChecked(settings.mainWindowGraphAveraged());


    if (settings.liveDataSourceType() == Settings::DS_TYPE_TEMPER) {
        ui->rbUsbSensor->setChecked(true);
        ui->cbLogToDb->setEnabled(true);
        ui->cbLogToDb->setChecked(settings.logToDatabase());
        ui->calibrationSpinBox->setEnabled(true);
        ui->calibrationSpinBox->setValue(settings.temperatureCalibrationOffset());
        ui->humidityCalibrationSpinBox->setEnabled(false);
        ui->gbCalibration->setEnabled(true);
        dataSourceChanged();
    } else if (settings.liveDataSourceType() == Settings::DS_TYPE_TEMPERHUM) {
        ui->rbTemperHumSensor->setChecked(true);
        ui->cbLogToDb->setEnabled(true);
        ui->calibrationSpinBox->setEnabled(true);
        ui->humidityCalibrationSpinBox->setEnabled(true);
        ui->gbCalibration->setEnabled(true);
        ui->cbLogToDb->setChecked(settings.logToDatabase());
        ui->calibrationSpinBox->setValue(settings.temperatureCalibrationOffset());
        ui->humidityCalibrationSpinBox->setValue(settings.humidityCalibrationOffset());
        dataSourceChanged();
    } else {
        ui->rbDatabase->setChecked(true);
        ui->cbLogToDb->setEnabled(false);
        ui->gbCalibration->setEnabled(false);
        dataSourceChanged();
    }

    // Database tab
    auto driverName = settings.databaseDriver();
    if (driverName == "QPSQL") {
        ui->cbDatabaseType->setCurrentIndex(PSQL);
        ui->stackedWidget->setCurrentIndex(ui->stackedWidget->indexOf(ui->pgPostgreSQL));
    } else {
        ui->cbDatabaseType->setCurrentIndex(SQLITE);
        ui->stackedWidget->setCurrentIndex(ui->stackedWidget->indexOf(ui->pgSQLite));
    }

    ui->hostnameLineEdit->setText(settings.databaseHostName());
    ui->portSpinBox->setValue(settings.databasePort());
    ui->databaseLineEdit->setText(settings.databaseName());
    ui->passwordLineEdit->setText(settings.databasePassword());
    ui->usernameLineEdit->setText(settings.databaseUsername());
    ui->leDatabaseFile->setText(settings.databaseFileName());

    dataSourceChanged();
}

void SettingsDialog::saveSettings() {
    Settings& settings = Settings::getInstance();

    // General tab
    settings.setMinimiseToSysTray(ui->cbMinimiseToSysTray->isChecked());
    settings.setCloseToSysTray(ui->cbMinimiseToSysTrayOnExit->isChecked());

    ChartColours cc;
    cc.temperature = ui->cbChartLineColour->color();
    cc.humidity = ui->cbHumidityChartLineColour->color();
    cc.dewPoint = ui->cbDewPointChartLineColour->color();
    settings.setChartColours(cc);

    ChartGraphs cg;
    cg.temperature = ui->cbPlotTemperature->isChecked();
    cg.humidity = ui->cbPlotHumidity->isChecked();
    cg.dewPoint = ui->cbPlotDewPoint->isChecked();
    settings.setMainWindowGraphsEnabled(cg);
    settings.setMainWindowGraphAveraged(ui->cbAveragedGraph->isChecked());

    settings.setMainWindowGraphLegendEnabled(ui->cbShowLegend->isChecked());

    // Data source tab
    if (ui->rbUsbSensor->isChecked()) {
        // TEMPer
        settings.setLiveDataSourceType(Settings::DS_TYPE_TEMPER);
        settings.setLogToDatabase(ui->cbLogToDb->isChecked());
        settings.setTemperatureCalibrationOffset(ui->calibrationSpinBox->value());
    } else if (ui->rbTemperHumSensor->isChecked()) {
        settings.setLiveDataSourceType(Settings::DS_TYPE_TEMPERHUM);
        settings.setLogToDatabase(ui->cbLogToDb->isChecked());
        settings.setTemperatureCalibrationOffset(ui->calibrationSpinBox->value());
        settings.setHumidityCalibrationOffset(ui->humidityCalibrationSpinBox->value());
    } else {
        settings.setLiveDataSourceType(Settings::DS_TYPE_DATABASE);
    }

    // Database tab
    if (ui->cbDatabaseType->currentIndex() == PSQL) {
        settings.setDatabaseDriver("QPSQL");
        settings.setDatabaseHostname(ui->hostnameLineEdit->text());
        settings.setDatabasePort(ui->portSpinBox->value());
        settings.setDatabaseName(ui->databaseLineEdit->text());
        settings.setDatabaseUsername(ui->usernameLineEdit->text());
        settings.setDatabasePassword(ui->passwordLineEdit->text());
    } else {
        settings.setDatabaseDriver("QSQLITE");
        auto fn = ui->leDatabaseFile->text();

        // A filename is required. If none was specified, make one up.
        if (fn.isEmpty()) {
            fn = QStandardPaths::locate(QStandardPaths::DataLocation,"", QStandardPaths::LocateDirectory);
            if (!fn.endsWith(QDir::separator()))
                fn.append(QDir::separator());
            fn.append("data.db");
        }

        settings.setDatabaseFileName(fn);
    }

}

void SettingsDialog::dataSourceChanged() {
    if (ui->rbUsbSensor->isChecked() || ui->rbTemperHumSensor->isChecked()) {
        ui->lblNoSqliteDataSource->setVisible(false);
        ui->leDatabaseFile->setEnabled(true);
        ui->tbBrowseForFile->setEnabled(true);
        ui->gbCalibration->setEnabled(true);
        ui->calibrationSpinBox->setEnabled(true);
        ui->cbLogToDb->setEnabled(true);
        ui->tabDatabase->setEnabled(ui->cbLogToDb->isChecked());

        ui->humidityCalibrationSpinBox->setEnabled(ui->rbTemperHumSensor->isChecked());
    } else {
        ui->lblNoSqliteDataSource->setVisible(true);
        ui->leDatabaseFile->setEnabled(false);
        ui->tbBrowseForFile->setEnabled(false);
        ui->tabDatabase->setEnabled(true);
        ui->gbCalibration->setEnabled(false);
        ui->cbLogToDb->setEnabled(false);
    }

    ui->lblHumidityOffset->setEnabled(ui->humidityCalibrationSpinBox->isEnabled());
}

void SettingsDialog::browseForDbFile() {
    auto fn = QFileDialog::getSaveFileName(this, tr("Select File"),
                                           QString(), tr("Database files") + " (*.db)");

    if (!fn.isEmpty())
        ui->leDatabaseFile->setText(fn);
}

void SettingsDialog::done() {

    if (ui->rbDatabase->isChecked() && (ui->cbDatabaseType->currentIndex() == SQLITE)) {
        QMessageBox::critical(this, tr("Configuration Error"),
                              tr("A SQLite database can not be used as a data source. Either "
                              "configure a PostgreSQL database or use a USB HID sensor device "
                              "as the data source."));
        return;
    }

    saveSettings();


    this->accept();
}
