/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "temperdatasource.h"

#include <QtDebug>
#include <QThread>

unsigned const char cmd_temp[9] = { 0x00, 0x01, 0x80, 0x33, 0x01, 0x00, 0x00, 0x00, 0x00 };
unsigned const char cmd_vers[9] = { 0x00, 0x01, 0x86, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x00 };
unsigned const char cmd_calb[9] = { 0x00, 0x01, 0x82, 0x77, 0x01, 0x00, 0x00, 0x00, 0x00 };

const int VENDOR_ID  = 0x0C45;
const int PRODUCT_ID = 0x7401;

TemperDataSource::TemperDataSource(float calibrationOffset, QObject *parent) :
    AbstractDataSource(parent)
{
    handle = NULL;
    collecting = false;

    this->calibrationOffset = calibrationOffset;

    timer.reset(new QTimer(this));
    connect(timer.data(), SIGNAL(timeout()), this, SLOT(collectData()));
}

TemperDataSource::~TemperDataSource() {
    stopCollecting();
    closeDevice();
}

bool TemperDataSource::openDevice() {
    //int res = hid_init();
    //qDebug() << "INIT:" << res;

    handle = hid_open(VENDOR_ID, PRODUCT_ID, NULL);

    if (handle == NULL) {
        qDebug() << "Device open failed.";
        emit error(tr("Error"),
                   tr("Failed to open USB HID sensor device. Check device is "
                   "connected\nand restart application."));
        return false;
    }

    double calibration = getCalibrationValue();
    qDebug() << calibration;

    QString version = getVersion();
    qDebug() << version;

    AbstractDataSource::openDevice();
    return true; // Ready for work.
}

bool TemperDataSource::closeDevice() {
    if (handle != NULL)
        hid_close(handle);
    collecting = false;
    return true;
}

void TemperDataSource::collectData() {
    emit newData(getTemperature());
}

// TODO: Is this really useful? There seems to be no way of setting
// the value stored on the device and by default none of the three
// sensors I have actually have a pre-set value.
double TemperDataSource::getCalibrationValue() const {
    unsigned char read_buffer[9];
    double calibration_value = 0;

    hid_write(handle, cmd_calb, sizeof(cmd_calb));
    QThread::usleep(100);
    hid_read(handle, read_buffer, 9);

    if (read_buffer[0] == 130) {
        calibration_value = read_buffer[2];
        calibration_value *= (1.0/16.0);
        qDebug() << "Calibration: " << calibration_value;
    }
    QThread::usleep(100);

    return calibration_value;
}

QString TemperDataSource::getVersion() const {
    unsigned char read_buffer[9];

    QString result;

    hid_write(handle, cmd_vers, sizeof(cmd_vers));
    QThread::usleep(100);
    hid_read(handle, read_buffer, 9);
    qDebug() << "Version read 1:" << read_buffer;
    result = QString::fromLocal8Bit((char*)read_buffer, 9);
    QThread::usleep(100);

    hid_read(handle, read_buffer, 9);
    qDebug() << "Version read 2:" << read_buffer;
    QThread::usleep(100);


    return result;
}

#define READ_BUFFER_SIZE 9

float TemperDataSource::getTemperature() const {
    int working = 0;
    float temperature = 0.0;
    unsigned char read_buffer[READ_BUFFER_SIZE];

    memset(read_buffer, 0, sizeof(char)*READ_BUFFER_SIZE);

    hid_write(handle, cmd_temp, sizeof(cmd_temp));
    QThread::usleep(50);
    hid_read(handle, read_buffer, 9);

    /*24.38  75.65*/

    working = (read_buffer[3] & 0xFF) + (read_buffer[2] << 8);
    temperature = working * (125.0/32000.0);
    if (temperature > 200.0)
        temperature = temperature - 256.0;

    //qDebug() << "Temp:" << temperature;

    return temperature + calibrationOffset;
}

void TemperDataSource::setInterval(int interval) {
    timer->setInterval(interval);
}

int TemperDataSource::getInterval() const {
    return timer->interval();
}

void TemperDataSource::startCollecting() {
    if (deviceOpen) {
        timer->start();
        collecting = true;
    }
}

void TemperDataSource::stopCollecting() {
    timer->stop();
    collecting = false;
}

bool TemperDataSource::isCollecting() {
    return collecting;
}
